/*

  Microsoft Windows Embedded Seminar 2007

    Module Name:
        Driver02.c

	Copyright(c) 2007 Yu Ito All rights reserved.

*/

#include <windows.h>
#include <Pkfuncs.h>

#include "..\Driver01\Drv01_cmd.h"
#include "Driver02.h"
#include "Drv02_cmd.h"

/*
	Debug Zones.
*/
#ifdef DEBUG

//
// These defines must match the ZONE_* defines in Driver02.h
//
DBGPARAM dpCurSettings = {
    TEXT("Driver02"), {
    TEXT("Errors"),TEXT("Warnings"),TEXT("Functions"),TEXT("Init"),
    TEXT("I/O"),TEXT("Undefined"),TEXT("Undefined"),TEXT("Undefined"),
    TEXT("Undefined"),TEXT("Undefined"),TEXT("Undefined"),TEXT("Undefined"),
    TEXT("Undefined"),TEXT("Undefined"),TEXT("Undefined"),TEXT("Undefined") },
	ZONE_INIT | ZONE_ERROR
};
#endif  // DEBUG

//
// Function prototype declarations
//

//
// External functions
//

//
// Global Variables
//

PDRV01_INFO		pHWHead;						// Pointer of Driver02 Information
LPBYTE			g_pMappedEmbedded;
LPBYTE			g_Marshalled;

BOOL WINAPI
DllEntry(
    HANDLE DllInstance,
    DWORD  Reason,
    LPVOID Reserved
    )
{
	switch ( Reason )
	{
	case DLL_PROCESS_ATTACH:
		DEBUGREGISTER( DllInstance );
		DEBUGMSG(ZONE_INIT, (TEXT("Driver02 driver : DLL Process Attach.\r\n")));
		break;

	case DLL_PROCESS_DETACH:
		DEBUGMSG(ZONE_INIT, (TEXT("Driver02 driver : DLL Process Detach.\r\n")));
		break;

	default:
		break;
    }
	return TRUE;
}

DWORD
DRV_Init(
    DWORD dwContext
    )
{
    LPTSTR			ActiveKey = (LPTSTR)dwContext;
	PDRV01_CONTEXT	pDrvContext;

    DEBUGMSG(ZONE_INIT, (TEXT("Driver02 driver : +DRV_Init\r\n")));

    pDrvContext = LocalAlloc(LPTR, sizeof(DRV01_CONTEXT));

	// Driver02 driver management area is allocated.
	pHWHead = VirtualAlloc(
					NULL,
					sizeof ( DRV01_INFO ),
					MEM_COMMIT,
					PAGE_READWRITE
					);
	if ( pHWHead == NULL ){
		DEBUGMSG( ZONE_ERROR, (TEXT("Driver02 driver :Failed to Allocate pHWHead.\r\n")));
		goto error_handler;
	}

	// Work area is allocated.
	pHWHead->pWorkBase = VirtualAlloc(
							NULL,
							DRIVER_WORK_SIZE,
							MEM_COMMIT,
							PAGE_READWRITE
							);
	if ( pHWHead->pWorkBase == NULL ){
		DEBUGMSG( ZONE_ERROR, (TEXT("Driver01 driver :Failed to Allocate pHWHead->WorkBase.\r\n")));
		goto error_handler;
	}

    DEBUGMSG(ZONE_INIT, (TEXT("Driver02 driver : -DRV_Init\r\n")));

    return (DWORD)pDrvContext;

error_handler:
	// Error handler

	if ( pHWHead->pWorkBase != NULL ){
		VirtualFree(
			pHWHead->pWorkBase,
			DRIVER_WORK_SIZE,
			MEM_DECOMMIT
			);
	}

	if ( pHWHead != NULL ){
		VirtualFree(
			pHWHead,
			sizeof ( DRV01_INFO ),
			MEM_DECOMMIT
			);
	}

    DEBUGMSG(ZONE_INIT, (TEXT("Driver02 driver : -DRV_Init\r\n")));

    return 0;
}   // DRV_Init

DWORD
DRV_Open(
    DWORD dwData,
    DWORD dwAccess,
    DWORD dwShareMode
    )
{
    PDRV01_CONTEXT pDrvContext = (PDRV01_CONTEXT)dwData;

    DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : +DRV_Open(0x%x)\r\n"),dwData));

	if ( pHWHead->OpenCount++ ){
		DEBUGMSG(ZONE_ERROR, (TEXT("Driver02 driver : too many open files.!!\r\n")));

		return (DWORD)NULL;
	}

    DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : -DRV_Open\r\n")));

    return (DWORD)pDrvContext;
}   // DRV_Open

BOOL
DRV_Close(
    DWORD Handle
    )
{
    BOOL bClose = TRUE;
    PDRV01_CONTEXT pDrvContext = (PDRV01_CONTEXT)Handle;

    DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : +DRV_Close\r\n")));

	if ( pHWHead->OpenCount ){
		pHWHead->OpenCount--;
	}
	else{
		DEBUGMSG(ZONE_ERROR, (TEXT("Driver02 driver : You have not opend any objects.\r\n")));

		bClose = FALSE;
	}

    DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : -DRV_Close\r\n")));

    return bClose;
}   // DRV_Close

BOOL
DRV_Deinit(
    DWORD dwContext
    )
{
	PDRV01_CONTEXT pDrvContext = (PDRV01_CONTEXT)dwContext;

    DEBUGMSG(ZONE_INIT, (TEXT("Driver02 driver : +DRV_Deinit\r\n")));

	if ( pHWHead->pWorkBase != NULL ){
		VirtualFree(
			pHWHead->pWorkBase,
			DRIVER_WORK_SIZE,
			MEM_DECOMMIT
			);
	}

	if ( pHWHead != NULL ){
		VirtualFree(
			pHWHead,
			sizeof ( DRV01_INFO ),
			MEM_DECOMMIT
			);
	}

    DEBUGMSG(ZONE_INIT, (TEXT("Driver02 driver : -DRV_Deinit\r\n")));

    LocalFree(pDrvContext);

    return TRUE;
}   // DRV_Deinit

BOOL
DRV_IOControl(
    DWORD Handle,
    DWORD dwIoControlCode,
    PBYTE pInBuf,
    DWORD nInBufSize,
    PBYTE pOutBuf,
    DWORD nOutBufSize,
    PDWORD pBytesReturned
    )
{
	BOOL	RetVal = TRUE;

    PDRV01_CONTEXT pDrvContext = (PDRV01_CONTEXT)Handle;

	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : +DRV_IOControl(%d)\r\n"), dwIoControlCode));

	switch ( dwIoControlCode ){
	case CMD_READ_1BYTE:
		{
			PDRV02_RWBUFFER	pBuffer = (PDRV02_RWBUFFER)pInBuf;
	
			if ( pBuffer->dwOffset < DRIVER_WORK_SIZE ){	// Data offset check
				*pBuffer->pData = *((LPBYTE)pHWHead->pWorkBase + pBuffer->dwOffset);
															// Exception !!
			}
			else{
				RetVal = FALSE;
			}
		}
		break;
	case CMD_WRITE_1BYTE:
		{
			PDRV02_RWBUFFER	pBuffer = (PDRV02_RWBUFFER)pInBuf;

			if ( pBuffer->dwOffset < DRIVER_WORK_SIZE ){	// Data offset check
				*((LPBYTE)pHWHead->pWorkBase + pBuffer->dwOffset) = *pBuffer->pData;
															// Exception !!
			}
			else{
				RetVal = FALSE;
			}
		}
		break;
	case CMD_DISPLAY_MESSAGE:
		MessageBox ( NULL, ( TEXT( "My name is Driver02.dll!!" )), ( TEXT( "Driver02" )), MB_OK );
		break;
	case CMD_CLEAR_BUFFER:
		{
			PDRV02_BUFFER	pBuffer = (PDRV02_BUFFER)pInBuf;
			DWORD	i;

			// Access check in Windows Embedded CE 6.0
	        if ( FAILED( CeOpenCallerBuffer(
							(PVOID *)&g_pMappedEmbedded,
							(LPVOID)pBuffer->pEmbeddedPointer,
							pBuffer->dwSize,
							ARG_O_PTR,
							TRUE)))
			{
				RetVal = FALSE;
				break;
			}

			ASSERT(g_pMappedEmbedded);

			// Marshaler in Windows Embedded CE 6.0
			if ( FAILED( CeAllocAsynchronousBuffer(
							(PVOID *)&g_Marshalled,
							(PVOID)g_pMappedEmbedded,
							pBuffer->dwSize,
							ARG_O_PTR
							)))
			{
				RetVal = FALSE;
				break;
			}

			for ( i = 0 ; i < DRIVER_WORK_SIZE ; i++ ){
//				*( g_pMappedEmbedded + i ) = 0x00;				// Synchronous
				*( g_Marshalled + i ) = 0x00;					// Asynchronous
			}

			if ( FAILED( CeFreeAsynchronousBuffer(
							(PVOID)g_Marshalled,
							(PVOID)g_pMappedEmbedded,
							pBuffer->dwSize,
							ARG_O_PTR
							)))
			{
				RetVal = FALSE;
				break;
			}

			if (FAILED( CeCloseCallerBuffer(
						(PVOID)g_pMappedEmbedded,
						(PVOID)pBuffer->pEmbeddedPointer,
						pBuffer->dwSize,
						ARG_O_PTR)))
			{
				ASSERT(!"Cleanup call to CeCloseCallerBuffer failed unexpectedly");
				RetVal = FALSE;
			}
		}
		break;
	case CMD_SET_KMBUFFER:
		{
			PDRV02_BUFFER	pBuffer = (PDRV02_BUFFER)pInBuf;
			HANDLE	hDrv01;
			DWORD	dwReturnValue;

			// Access check in Windows Embedded CE 6.0
	        if ( FAILED( CeOpenCallerBuffer(
							(PVOID *)&g_pMappedEmbedded,
							(LPVOID)pBuffer->pEmbeddedPointer,
							pBuffer->dwSize,
							ARG_O_PTR,
							TRUE)))
			{
				RetVal = FALSE;
				break;
			}

			ASSERT(g_pMappedEmbedded);

			// Driver Driver01 open
			hDrv01 = CreateFile (
						TEXT("DRV1:"),
						GENERIC_READ | GENERIC_WRITE,
						0,
						NULL,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL,
						NULL
						);
			if ( !hDrv01 ) {					// don't open
				RETAILMSG (1, (TEXT("Driver01 open function is failed.\r\n")));
				return 0;
			}

			// Write Kernel Mode Heap
			DeviceIoControl(
				hDrv01,
				CMD_WRITE_HEAP,
				(LPVOID)g_pMappedEmbedded,
				sizeof(LPVOID),
				NULL,
				0,
				&dwReturnValue,
				NULL
				);

			CloseHandle ( hDrv01 );

			if (FAILED( CeCloseCallerBuffer(
						(PVOID)g_pMappedEmbedded,
						(PVOID)pBuffer->pEmbeddedPointer,
						pBuffer->dwSize,
						ARG_O_PTR)))
			{
				ASSERT(!"Cleanup call to CeCloseCallerBuffer failed unexpectedly");
				RetVal = FALSE;
			}
		}
		break;
	default:
		break;
	}


	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : -DRV_IOControl\r\n")));

    return RetVal;
}   // DRV_IOControl

DWORD
DRV_Read(
	DWORD Handle,
	LPVOID pBuffer,
	DWORD dwNumBytes
	)
{
	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : +DRV_Read\r\n")));

	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : -DRV_Read\r\n")));

	return 0;
}	// DRV_Read

DWORD
DRV_Write(
	DWORD Handle,
	LPCVOID pBuffer,
	DWORD dwNumBytes
	)
{
	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : +DRV_Write\r\n")));

	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : -DRV_Write\r\n")));

	return 0;
}

DWORD
DRV_Seek(
	DWORD Handle,
	long lDistance,
	DWORD dwMoveMethod
	)
{
	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : +DRV_Seek\r\n")));



	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : -DRV_Seek\r\n")));

	return 0;
}

void
DRV_PowerUp(
	void
	)
{
	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : +DRV_PowerUp\r\n")));

	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : -DRV_PowerUp\r\n")));

	return;
}

void
DRV_PowerDown(
	void
	)
{
	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : +DRV_PowerDown\r\n")));

	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver02 driver : -DRV_PowerDown\r\n")));

	return;
}
