//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this source code is subject to the terms of the Microsoft end-user
// license agreement (EULA) under which you licensed this SOFTWARE PRODUCT.
// If you did not accept the terms of the EULA, you are not authorized to use
// this source code. For a copy of the EULA, please see the LICENSE.RTF on your
// install media.
//
////////////////////////////////////////////////////////////////////////////////
//
//  TuxDriver01 TUX DLL
//
//  Module: test.cpp
//          Contains the test functions.
//
//  Revision History:
//
////////////////////////////////////////////////////////////////////////////////

#include "main.h"
#include "globals.h"
#include "Drv01_cmd.h"


////////////////////////////////////////////////////////////////////////////////
// TestProc 1
//  Executes one test.
//
// Parameters:
//  uMsg            Message code.
//  tpParam         Additional message-dependent data.
//  lpFTE           Function table entry that generated this call.
//
// Return value:
//  TPR_PASS if the test passed, TPR_FAIL if the test fails, or possibly other
//  special conditions.
////////////////////////////////////////////////////////////////////////////////

TESTPROCAPI TestProc1(UINT uMsg, TPPARAM tpParam, LPFUNCTION_TABLE_ENTRY lpFTE)
{
	TESTPROCAPI	tRet=TPR_PASS;
	HANDLE		hDrv01;
	DWORD		dwReturnValue;
	DWORD		dwValue;

	// The shell doesn't necessarily want us to execute the test. Make sure
    // first.
    if(uMsg != TPM_EXECUTE)
    {
        return TPR_NOT_HANDLED;
    }

    g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll IoControl test Starting"));

    g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - Open Driver01.dll"));
	// Driver Driver01 open
	hDrv01 = CreateFile (
				TEXT("DRV1:"),
				GENERIC_READ | GENERIC_WRITE,
				0,
				NULL,
				OPEN_EXISTING,
				FILE_ATTRIBUTE_NORMAL,
				NULL
				);
	if ( hDrv01 == INVALID_HANDLE_VALUE ) {
		g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - Failed to open Driver01.dll"));
		tRet = TPR_FAIL;
	}
	else{
		g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - Open Driver01.dll OK"));

		// CMD_IOCTL_NO1
		if ( DeviceIoControl( hDrv01, CMD_IOCTL_NO1, NULL, 0, &dwValue, sizeof(DWORD), &dwReturnValue, NULL ) ){
			if ( dwValue == 1 ){
				g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO1 OK"));
			}
			else{
				g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO1 is invalid return value"));
				tRet = TPR_FAIL;
			}
		}
		else{
			g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO1 is FAILED"));
			tRet = TPR_FAIL;
		}

		// CMD_IOCTL_NO2
		if ( DeviceIoControl( hDrv01, CMD_IOCTL_NO2, NULL, 0, &dwValue, sizeof(DWORD), &dwReturnValue, NULL ) ){
			if ( dwValue == 2 ){
				g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO2 OK"));
			}
			else{
				g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO2 is invalid return value"));
				tRet = TPR_FAIL;
			}
		}
		else{
			g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO2 is FAILED"));
			tRet = TPR_FAIL;
		}

		// CMD_IOCTL_NO3
		if ( DeviceIoControl( hDrv01, CMD_IOCTL_NO3, NULL, 0, &dwValue, sizeof(DWORD), &dwReturnValue, NULL ) ){
			if ( dwValue == 3 ){
				g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO3 OK"));
			}
			else{
				g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO3 is invalid return value"));
				tRet = TPR_FAIL;
			}
		}
		else{
			g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO3 is FAILED"));
			tRet = TPR_FAIL;
		}

		// CMD_IOCTL_NO4
		if ( DeviceIoControl( hDrv01, CMD_IOCTL_NO4, NULL, 0, &dwValue, sizeof(DWORD), &dwReturnValue, NULL ) ){
			if ( dwValue == 4){
				g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO4 OK"));
			}
			else{
				g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO4 is invalid return value"));
				tRet = TPR_FAIL;
			}
		}
		else{
			g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - CMD_IOCTL_NO4 is FAILED"));
			tRet = TPR_FAIL;
		}

		CloseHandle ( hDrv01 );
	}

	return tRet;
}

////////////////////////////////////////////////////////////////////////////////
// TestProc 2
//  Executes one test.
//
// Parameters:
//  uMsg            Message code.
//  tpParam         Additional message-dependent data.
//  lpFTE           Function table entry that generated this call.
//
// Return value:
//  TPR_PASS if the test passed, TPR_FAIL if the test fails, or possibly other
//  special conditions.
////////////////////////////////////////////////////////////////////////////////

TESTPROCAPI TestProc2(UINT uMsg, TPPARAM tpParam, LPFUNCTION_TABLE_ENTRY lpFTE)
{
	TESTPROCAPI	tRet=TPR_PASS;
	HANDLE		hDrv01_1st;
	HANDLE		hDrv01_2nd;

	// The shell doesn't necessarily want us to execute the test. Make sure
    // first.
    if(uMsg != TPM_EXECUTE)
    {
        return TPR_NOT_HANDLED;
    }

    g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Open test Starting"));

    g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - Open Driver01.dll (1st)"));
	// Driver Driver01 open (1st)
	hDrv01_1st = CreateFile (
					TEXT("DRV1:"),
					GENERIC_READ | GENERIC_WRITE,
					0,
					NULL,
					OPEN_EXISTING,
					FILE_ATTRIBUTE_NORMAL,
					NULL
					);
	if ( hDrv01_1st == INVALID_HANDLE_VALUE ) {
		g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - Failed to open Driver01.dll (1st)"));
		tRet = TPR_FAIL;
	}
	else{
		g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - Open Driver01.dll OK (1st)"));

	    g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - Open Driver01.dll (2nd)"));
		// Driver Driver01 open (2nd)
		hDrv01_2nd = CreateFile (
						TEXT("DRV1:"),
						GENERIC_READ | GENERIC_WRITE,
						0,
						NULL,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL,
						NULL
						);
		if ( hDrv01_2nd != INVALID_HANDLE_VALUE ) {
			g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - Failed to multiple open Driver01.dll"));
			tRet = TPR_FAIL;
		}
		else{
			g_pKato->Log(LOG_COMMENT, TEXT("Driver01.dll Test - Driver01.dll don't support multiple open"));
		}

		CloseHandle ( hDrv01_1st );
	}

	return tRet;
}
