/*

  Copyright(c) 2006 Yu Ito All rights reserved.

    Module Name:

        Driver01.c

    Revision History:

		2007.2.4		Released

*/

#include <windows.h>

#include "Driver01.h"

/*
	Debug Zones.
*/
#ifdef DEBUG

//
// These defines must match the ZONE_* defines in Driver01.h
//
DBGPARAM dpCurSettings = {
    TEXT("Drviver01"), {
    TEXT("Errors"),TEXT("Warnings"),TEXT("Functions"),TEXT("Init"),
    TEXT("I/O"),TEXT("Undefined"),TEXT("Undefined"),TEXT("Undefined"),
    TEXT("Undefined"),TEXT("Undefined"),TEXT("Undefined"),TEXT("Undefined"),
    TEXT("Undefined"),TEXT("Undefined"),TEXT("Undefined"),TEXT("Undefined") },
	ZONE_INIT | ZONE_ERROR
};
#endif  // DEBUG

//
// Function prototype declarations
//

//
// External functions
//

//
// Global Variables
//

PDRV01_INFO		pHWHead;						// Pointer of Driver01 Information

BOOL WINAPI
DllEntry(
    HANDLE DllInstance,
    DWORD  Reason,
    LPVOID Reserved
    )
{
	switch ( Reason )
	{
	case DLL_PROCESS_ATTACH:
		DEBUGREGISTER( DllInstance );
		DEBUGMSG(ZONE_INIT, (TEXT("Driver01 driver : DLL Process Attach.\r\n")));
		break;

	case DLL_PROCESS_DETACH:
		DEBUGMSG(ZONE_INIT, (TEXT("Driver01 driver : DLL Process Detach.\r\n")));
		break;
    case DLL_THREAD_ATTACH:
		DEBUGMSG(ZONE_INIT, (TEXT("Driver01 driver : DLL Thread Attach.\r\n")));
        break;
    case DLL_THREAD_DETACH:
		DEBUGMSG(ZONE_INIT, (TEXT("Driver01 driver : DLL Thread Detach.\r\n")));
        break;
	default:
		break;
    }
	return TRUE;
}

DWORD
DRV_Init(
    DWORD dwContext
    )
{
    LPTSTR			ActiveKey = (LPTSTR)dwContext;
	PDRV01_CONTEXT	pDrvContext;

    DEBUGMSG(ZONE_INIT, (TEXT("Driver01 driver : +DRV_Init\r\n")));

    pDrvContext = LocalAlloc(LPTR, sizeof(DRV01_CONTEXT));

	// Driver01hCoǗ̈mۂ܂B
	pHWHead = VirtualAlloc(
					NULL,
					sizeof ( DRV01_INFO ),
					MEM_COMMIT,
					PAGE_READWRITE
					);
	if ( pHWHead == NULL ){
		DEBUGMSG( ZONE_ERROR, (TEXT("Driver01 driver :Failed to Allocate pHWHead.\r\n")));
		goto error_handler;
	}

    DEBUGMSG(ZONE_INIT, (TEXT("Driver01 driver : -DRV_Init\r\n")));

    return (DWORD)pDrvContext;

error_handler:

	// G[nh
	if ( pHWHead != NULL ){

		VirtualFree(
			pHWHead,
			sizeof ( DRV01_INFO ),
			MEM_DECOMMIT
			);
	}

    DEBUGMSG(ZONE_INIT, (TEXT("Driver01 driver : -DRV_Init\r\n")));

    return 0;
}   // DRV_Init

DWORD
DRV_Open(
    DWORD dwData,
    DWORD dwAccess,
    DWORD dwShareMode
    )
{
    PDRV01_CONTEXT pDrvContext = (PDRV01_CONTEXT)dwData;

    DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : +DRV_Open(0x%x)\r\n"),dwData));

	if ( pHWHead->OpenCount++ ){
		DEBUGMSG(ZONE_ERROR, (TEXT("Driver01 driver : too many open files.!!\r\n")));

		return (DWORD)NULL;
	}

    DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : -DRV_Open\r\n")));

    return (DWORD)pDrvContext;
}   // DRV_Open

BOOL
DRV_Close(
    DWORD Handle
    )
{
    BOOL bClose = TRUE;
    PDRV01_CONTEXT pDrvContext = (PDRV01_CONTEXT)Handle;

    DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : +DRV_Close\r\n")));

	if ( pHWHead->OpenCount ){
		pHWHead->OpenCount--;
	}
	else{
		DEBUGMSG(ZONE_ERROR, (TEXT("Driver01 driver : You have not opend any objects.\r\n")));

		bClose = FALSE;
	}

    DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : -DRV_Close\r\n")));

    return bClose;
}   // DRV_Close

BOOL
DRV_Deinit(
    DWORD dwContext
    )
{
	PDRV01_CONTEXT pDrvContext = (PDRV01_CONTEXT)dwContext;

    DEBUGMSG(ZONE_INIT, (TEXT("Driver01 driver : +DRV_Deinit\r\n")));

	if ( pHWHead != NULL ){
		VirtualFree(
			pHWHead,
			sizeof ( DRV01_INFO ),
			MEM_DECOMMIT
			);
	}

	if ( pHWHead->hThread != NULL ){
		CloseHandle ( pHWHead->hThread );
	}

    DEBUGMSG(ZONE_INIT, (TEXT("Driver01 driver : -DRV_Deinit\r\n")));

    LocalFree(pDrvContext);

    return TRUE;
}   // DRV_Deinit

BOOL
DRV_IOControl(
    DWORD Handle,
    DWORD dwIoControlCode,
    PBYTE pInBuf,
    DWORD nInBufSize,
    PBYTE pOutBuf,
    DWORD nOutBufSize,
    PDWORD pBytesReturned
    )
{
	BOOL	RetVal = TRUE;

    PDRV01_CONTEXT pDrvContext = (PDRV01_CONTEXT)Handle;

	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : +DRV_IOControl(%d)\r\n"), dwIoControlCode));
	// R}h𔻒肵܂B
	switch ( dwIoControlCode ){
	default:
		break;
	}


	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : -DRV_IOControl\r\n")));

    return RetVal;
}   // DRV_IOControl

DWORD
DRV_Read(
	DWORD Handle,
	LPVOID pBuffer,
	DWORD dwNumBytes
	)
{
	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : +DRV_Read\r\n")));

	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : -DRV_Read\r\n")));

	return 0;
}	// DRV_Read

DWORD
DRV_Write(
	DWORD Handle,
	LPCVOID pBuffer,
	DWORD dwNumBytes
	)
{
	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : +DRV_Write\r\n")));

	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : -DRV_Write\r\n")));

	return 0;
}

DWORD
DRV_Seek(
	DWORD Handle,
	long lDistance,
	DWORD dwMoveMethod
	)
{
	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : +DRV_Seek\r\n")));



	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : -DRV_Seek\r\n")));

	return 0;
}

void
DRV_PowerUp(
	void
	)
{
	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : +DRV_PowerUp\r\n")));

	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : -DRV_PowerUp\r\n")));

	return;
}

void
DRV_PowerDown(
	void
	)
{
	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : +DRV_PowerDown\r\n")));

	DEBUGMSG(ZONE_FUNCTION, (TEXT("Driver01 driver : -DRV_PowerDown\r\n")));

	return;
}
